
from io import BytesIO
from urllib import request

import numpy as np
from PIL import Image


def download_image(url):
    with request.urlopen(url) as resp:
        buffer = resp.read()
    stream = BytesIO(buffer)
    img = Image.open(stream)
    return img


def default_prepare_image(img, target_size=(224, 224)):
    if img.mode != 'RGB':
        img = img.convert('RGB')
    img = img.resize(target_size, Image.NEAREST)
    return img


class BasePreprocessor:
    def __init__(self, target_size):
        self.target_size = target_size
    
    def resize_image(self, img):
        return default_prepare_image(img, self.target_size)

    def image_to_array(self, img):
        return np.array(img, dtype='float32')

    def preprocess(self, X):
        raise Exception('not implemented')

    def convert_to_tensor(self, img):
        small = self.resize_image(img)
        x = self.image_to_array(small)
        batch = np.expand_dims(x, axis=0)
        return self.preprocess(batch)

    def from_path(self, path):
        with Image.open(path) as img:
            return self.convert_to_tensor(img)

    def from_url(self, url):
        img = download_image(url)
        return self.convert_to_tensor(img)
    





from keras_image_helper.base import BasePreprocessor


def tf_preprocessing(x):
    x /= 127.5
    x -= 1.0
    return x


def caffe_preprocessing(x):
    # 'RGB'->'BGR'
    x = x[..., ::-1]

    mean = [103.939, 116.779, 123.68]

    x[..., 0] -= mean[0]
    x[..., 1] -= mean[1]
    x[..., 2] -= mean[2]

    return x


class ResnetPreprocessor(BasePreprocessor):
    # sources:
    # 
    # https://github.com/keras-team/keras-applications/blob/master/keras_applications/xception.py
    #   preprocess_input: 
    #      imagenet_utils.preprocess_input(x, mode='tf', **kwargs)
    # 
    # https://github.com/keras-team/keras-applications/blob/master/keras_applications/imagenet_utils.py
    #   _preprocess_numpy_input, mode == 'tf'
    # 

    def preprocess(self, X):
        return caffe_preprocessing(X)


class XceptionPreprocessor(BasePreprocessor):
    # sources:
    # 
    # https://github.com/keras-team/keras-applications/blob/master/keras_applications/xception.py
    #   preprocess_input: 
    #      imagenet_utils.preprocess_input(x, mode='tf', **kwargs)
    # 
    # https://github.com/keras-team/keras-applications/blob/master/keras_applications/imagenet_utils.py
    #   _preprocess_numpy_input, mode == 'tf'
    # 

    def preprocess(self, x):
        return tf_preprocessing(x)


class VGGPreprocessor(BasePreprocessor):
    # sources:
    # 
    # https://github.com/keras-team/keras-applications/blob/master/keras_applications/vgg16.py
    #   preprocess_input = imagenet_utils.preprocess_input
    #
    # https://github.com/keras-team/keras-applications/blob/master/keras_applications/imagenet_utils.py
    #   _preprocess_numpy_input, mode == 'caffe'
    # 

    def preprocess(self, x):
        return caffe_preprocessing(x)


class InceptionPreprocessor(BasePreprocessor):
    # sources:
    # 
    # https://github.com/keras-team/keras-applications/blob/master/keras_applications/inception_v3.py
    #    imagenet_utils.preprocess_input(x, mode='tf', **kwargs)
    #
    # https://github.com/keras-team/keras-applications/blob/master/keras_applications/imagenet_utils.py
    #   _preprocess_numpy_input, mode == 'tf'
    # 

    def preprocess(self, x):
        return tf_preprocessing(x)


import pathlib
from setuptools import setup

from keras_image_helper import __version__

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="keras_image_helper",
    version=__version__,
    description="A lightweight library for pre-processing images for pre-trained keras models",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/alexeygrigorev/keras-image-helper",
    author="Alexey Grigorev",
    author_email="alexey@datatalks.club",
    license="WTFPL",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
    ],
    packages=["keras_image_helper"],
    include_package_data=True,
    install_requires=["numpy", "pillow"],
)


from keras_image_helper.preprocessors import ResnetPreprocessor
from keras_image_helper.preprocessors import XceptionPreprocessor
from keras_image_helper.preprocessors import VGGPreprocessor
from keras_image_helper.preprocessors import InceptionPreprocessor


# reference: https://keras.io/api/applications/

preprocessors = {
    'xception': XceptionPreprocessor,
    'resnet50': ResnetPreprocessor,
    'vgg16': VGGPreprocessor,
    'inception_v3': InceptionPreprocessor,
}


def create_preprocessor(name, target_size, **params):
    name = name.lower()
    if name not in preprocessors:
        raise Exception('Unknown model %s' % name)
    Preprocessor = preprocessors[name]
    return Preprocessor(target_size=target_size, **params)